#! @perlpath@

# uses tables:
# purpose,last,pop,as_ref,aut_num
#################################

# --> [MODULES]
use strict;                          # Default
use DBI;                             # Database         http://search.cpan.org/search?dist=DBI
use Net::IP;                         # IP subs          http://search.cpan.org/search?dist=Net-IP
use Math::BigInt;                    # IP calcs         http://search.cpan.org/search?dist=Math-BigInt
use Term::ANSIColor qw(:constants);  # Coloured output  http://search.cpan.org/search?dist=ANSIColor
use Net::RIPEWhois::in;              # Ripe Whois       from package ripe-asused

my $script_vers = "2.1";
my $script_date = "2004-05-07";

# --> [GLOBAL_VARS]
use vars qw($tmp_dbh @glb_purpose @glb_location
             @glb_purpose_ext @glb_location_ext
             @ip_data_lp @ip_data $my_back @tmp_template
             $my_debug @my_warnings
             $my_range_start $my_range_stop $my_range_start_b $my_range_stop_b
             @my_ranges
           );

# --> [INIT]
BEGIN: {
    # Commandline parsing... [prepare]
    my $tmp_commands = join(' ',@ARGV);
    @ARGV = split(/ \-/,$tmp_commands);
    undef($tmp_commands);
    }

&main;
exit;

##############################################################
##############################################################
sub main {
# --> [MAIN]
    my ($my_purpose, $my_location, $my_sizeincidr, $my_sizeincidr_org, $my_templ_netname,$my_range, $my_list, $my_regid, $my_regid_file);
    my ($r_start,$r_end,$r_pur,$r_loc,$r_status,$line,$x,$r_last_used_start,$r_last_used_end,$change_templ);
    my ($my_cust, $my_cust_free);

    my $my_cmd        = 1;                          # 1: help | 2: view-purposes | 3: view-locations | 4: other
    my $my_cmd_sub    = 0;                          # 0: free | 1: used | 2: both | 3: none
    my $my_show       = 1;
    my $my_config     = '@confdir@/aisst.conf';     # default config-path
    my $my_split      = 0;                          # splitting
    $my_back          = 0;                          # reverse output
    $my_debug         = 0;                          # 1: enable debug-mode
    my $my_templ_save = 0;                          # 0 -> no_outout | 1 -> file | 2 -> stdout
    my $my_short      = 1;                          # 1 -> default | 2 -> summary | 3 -> range_stats
    my $my_free_all   = 1;
    my $my_connect    = 1;                          # MySQL-Connection: 0: disabled | 1: enabled

    my $SOURCE="EUCW";

    for (@ARGV) {
        $_ =~ s/^\-//;
        if    (/^purposes$/)        {$my_cmd=2; last}
        elsif (/^facility$/)        {$my_cmd=3; last}
        elsif (/^debug( (\d))?$/)   {if ($2 =~ /^\d$/) {$my_debug=$2} else {$my_debug=1}}

        elsif (/cl (\d+.+)/)        {$my_range=$1; $my_cmd=4; $my_cmd_sub=0; $my_show=-1; $my_connect=0; last}
        elsif (/cl (\D+.+)/)        {$my_regid=$1; $my_cmd=4; $my_cmd_sub=0; $my_show=-1; $my_connect=0; last}
        elsif (/regid (.+)$/)       {$my_regid_file = $1; $my_cmd=4; $my_cmd_sub=2; $my_show=-1; last}
        elsif (/^p (.*)$/)          {$my_purpose = uc($1); $my_cmd=4}
        elsif (/^f (.*)$/)          {$my_location = uc($1); $my_cmd=4}

        elsif (/^size \/?(\d+)$/)   {$my_sizeincidr = $1; $my_sizeincidr_org = $my_sizeincidr; $my_free_all=1}
        elsif (/^config (.*)/)      {$my_config=$1}
        elsif (/^show (\d+)|(all)/) {if ($2 eq 'all') {$my_show=-1} else {$my_show=$1}}
        elsif (/^t( (.*))?$/)       {$my_templ_netname=$2;}
        elsif (/^s$/)               {$my_split=1; die "-size required before this argument" if (!$my_sizeincidr); $my_sizeincidr++; $my_show=2; $my_free_all=1}
        elsif (/^b$/)               {$my_back=1}
        elsif (/^list( (.+))?$/)    {
            $my_cmd=4; $my_cmd_sub=2; $my_show=-1;
            if ($2 ne '') {$my_connect=0; $my_list=$2}
        }
        elsif (/^free( (\d+))?$/)   {$my_cmd=4; $my_cmd_sub=0; $my_show=-1; $my_show=$2 if (defined($2))}
        elsif (/^assigned$/)        {$my_cmd=4; $my_cmd_sub=1; $my_show=-1}
        elsif (/^o( (.+))?$/)       {$my_templ_save=1; if ($1 ne '') {$change_templ=">$1"}}
        elsif (/^oo( (.+))?$/)      {$my_templ_save=1; if ($1 ne '') {$change_templ=">>$1"}}
        elsif (/^rs$/)              {$my_short=3; $my_cmd=4; $my_cmd_sub=3; $my_show=-1}

        elsif (/^sum$/)             {$my_short=2; $my_cmd_sub=3; $my_show=-1}
        elsif (/^n (\S+)$/)         {$my_cust = $1}

        elsif (/^\-?help$/)         {$my_cmd=1}

        else {die "Unknown Argument: $_ (try -help)"}
    }

# --> [LOAD_CONF]
    require $my_config;
    $Conf::TEMPL_OUT = $change_templ if $change_templ;

    if ($my_cmd==1) {
        &usage;
    }

    if ( ($my_templ_netname) or ($my_show==1) ) {
        if (!$my_templ_netname) {
            $my_templ_netname=$Conf::TEMPL_NETNAME;
        }
        if (-f "$ENV{'HOME'}/.inetnum") {
            open(TEMPL,"$ENV{'HOME'}/.inetnum") || &message($Conf::msg[7],9)
        }
        elsif (-f "~ipadm/.inetnum") {
            open(TEMPL,"~ipadm/.inetnum") || &message($Conf::msg[7],9)
        }
        else {
            open(TEMPL,".inetnum") || &message($Conf::msg[7],9,$!)
        }
        @tmp_template = <TEMPL>;
        close(TEMPL);
        if (($my_show==1) and ($my_templ_save != 1)) {
            $my_templ_save=2;
        }
    }

# --> [CONNECT]
    $tmp_dbh = DBI->connect($Conf::SQLCONF, $Conf::SQLUSER, $Conf::SQLPASS) || &message($Conf::msg[1],9) if ($my_connect==1);

    my ($tmp_query,$tmp_whois);

# --> [INFO]
    if    ($my_cmd == 2) {&show_purposes}
    elsif ($my_cmd == 3) {&show_facility}
    elsif ($my_cmd == 4) {
        my $tmp_whois = new RipeWhois('Host'          => $Conf::WHOISHOST,
                                      'Port'          => $Conf::WHOISPORT,
                                      'FormatMode'    => 1
        );

        # --> [CHECK_PL]
        if ($my_purpose) {
            &sql_query_load(1);
            my $tmp_in=0;
            for (@glb_purpose) {
                if ($my_purpose eq $_) {
                    $tmp_in=1;
                }
            }
            &message($Conf::msg[4],9,$my_purpose) unless $tmp_in;
        }
        if ($my_location) {
            &sql_query_load(2);
            my $tmp_in=0;
            for (@glb_location) {
                if ($my_location eq $_) {
                    $tmp_in=1;
                }
            }
            &message($Conf::msg[5],9,$my_location) unless $tmp_in;
        }
    
        print $Conf::msg[51]; # here "\n";

        my $tmp_query;
        my @ip_request;

# --> [QUERY_PL]
# query | purpose
        if ( ( $my_purpose ) and (!$my_location) ) {
            $tmp_whois->Query("-s $SOURCE -i usedfor $my_purpose");
            $tmp_query=$tmp_whois->GetResult();
            &read_out($tmp_query);
            push @ip_request, @ip_data_lp;
        }

# query | facility
        if ( ($my_location) and (!$my_purpose) ) {
            $tmp_whois->Query("-s $SOURCE -i location $my_location");
            $tmp_query=$tmp_whois->GetResult();
            &read_out($tmp_query);
            push @ip_request, @ip_data_lp;
        }

# query | both
        if ( ($my_location) and ($my_purpose) ) {
            $tmp_whois->Query("-s $SOURCE -i usedfor $my_purpose");
            $tmp_query=$tmp_whois->GetResult();
            &read_out($tmp_query);
    
            my $tmp_location;

            for (@ip_data_lp) {
                (undef,undef,undef,undef,$tmp_location,undef) = split(/\|/,$_);
                if ($tmp_location =~ /^$my_location$/i) {
                    push @ip_request, $_;
                }
            }
        }

# option -cl <IP>
        if ($my_range =~ /([^\/]+)\/(\d+)/) {
            my ($tmp_a,$tmp_b) = Net::IP::ip_prefix_to_range($1,$2,4);
            push @ip_request, "$tmp_a|$tmp_b|0|||0|";

            $my_range_start_b = Net::IP::ip_iptobin($tmp_a,4);
            $my_range_stop_b  = Net::IP::ip_iptobin($tmp_b,4);

            $my_range_start   = Math::BigInt->new(Net::IP::ip_bintoint($my_range_start_b));
            $my_range_stop    = Math::BigInt->new(Net::IP::ip_bintoint($my_range_stop_b));
        }

        if ($my_list =~ /([^\/]+)\/(\d+)/) {
            my ($tmp_a,$tmp_b) = Net::IP::ip_prefix_to_range($1,$2,4);
            push @ip_request, "$tmp_a|$tmp_b|0|||0|";

            $my_range_start_b = Net::IP::ip_iptobin($tmp_a,4);
            $my_range_stop_b  = Net::IP::ip_iptobin($tmp_b,4);

            $my_range_start = Math::BigInt->new(Net::IP::ip_bintoint($my_range_start_b));
            $my_range_stop  = Math::BigInt->new(Net::IP::ip_bintoint($my_range_stop_b));
        }

# option -cl <REG-ID>
        if ($my_regid ne '') {
            $tmp_whois->Query("-s $SOURCE -r -Tinetnum -i registry $my_regid");
            $tmp_query=$tmp_whois->GetResult();
            my @tmp_ip_data=&read_out_inet($tmp_query,,1);
            for (@tmp_ip_data) {
                my (undef,$tmp_a,$tmp_b,undef) = split(/\|/,$_,4);
                push @ip_request, "$tmp_a|$tmp_b|0|||0|";
            }
        }

# option -cl <REG-ID> as file!!
        if ($my_regid_file ne '') {
            $my_regid = '*';
            die "File not found! $my_regid_file" if (!-f $my_regid_file);

            open(RID,$my_regid_file) || die "Can't open $my_regid_file!";
            my @in = <RID>;
            close(RID);

            for (@in) {
                if (/^alloc\s+\=\s+([^\/]+)\/(\d+)\s+$/i) {
                    my ($tmp_a,$tmp_b) = Net::IP::ip_prefix_to_range($1,$2,4);
                    push @ip_request, "$tmp_a|$tmp_b|0|||0|";
                }
            }
        }

        # --> [SORT_IP_REQ]
        if ($my_back) {
            print $Conf::msg[53];
            @ip_request=reverse @ip_request;
        }
        else {
        # order by pref
            my $tmp_a_pref;
            my $tmp_b_pref;
            @ip_request = sort {
                (undef,undef,$tmp_a_pref,undef)=split(/\|/,$a,4);
                (undef,undef,$tmp_b_pref,undef)=split(/\|/,$b,4);
                $tmp_a_pref <=> $tmp_b_pref;
            } @ip_request;
        }

        my ($tmp_range_start, $tmp_range_end, $tmp_pref, $tmp_purpose, $tmp_location, $tmp_used, $my_used, $tmp_netname, @tmp_ip_data, $tmp_ip_pointer);
        my $tmp_query;
        my $count_real_ranges=0;

        if (!@ip_request) {
            &message($Conf::msg[6],9);
        }

# --> [TEMPL]
        if ($my_templ_save==1) {
            open(TEMPL,"$Conf::TEMPL_OUT");
        }
        my @tmp_template_copy;

        if ($my_cust ne '') {
            if ($my_debug) {
                print "DEBUG_MSG: [assignment_check] netname: $my_cust\n";
            }

            # Date/Time
            my (undef, undef, $t_std, $t_tag, $t_mon, $t_jahr, undef, undef, undef) = localtime(time);
            $t_mon++;
            if ($t_mon == 13) {
                $t_mon = 1;
            }

            $t_jahr += 1900;
            for ($t_std, $t_tag, $t_mon) {
                $_ =~ s/^(\d)$/0$1/;
            }

            my $my_now   = "$t_jahr$t_mon$t_tag"; $t_jahr--;
            my $my_lyear = "$t_jahr$t_mon$t_tag";

            $tmp_whois->Query("-s $SOURCE -r -T inetnum $my_cust");
            my @my_cust_query=split(/\n/,$tmp_whois->GetResult());
            my ($tmp_a,$tmp_b,$tmp_changed);
            $my_cust_free = 0;

            my ($tmp_c,$tmp_firstinet);
            for (@my_cust_query) {
                $tmp_c++; next if (/^\%/);

                if (/^inetnum:\s+(\d+\.\d+\.\d+\.\d+) \- (\d+\.\d+\.\d+\.\d+)\s*$/) {
                    $tmp_a=$1; $tmp_b=$2; $tmp_changed = $tmp_changed;
                    $tmp_firstinet="$1 - $2" if ($tmp_firstinet eq '');
                }
                elsif (/^assigned:\s+(\d+)\.(\d+)\.(\d+)/) {
                    $tmp_changed = "20$3$2$1";
                }
                elsif (/^changed:\s+\S+\s+(\d{4}\d{2}\d{2})/) {
                    if ($1 < $tmp_changed) {
                        $tmp_changed = $1;
                    }
                }
                if (((/^\s*$/) or (@my_cust_query==$tmp_c)) && ($tmp_a ne '')) {
                    if ($tmp_changed > $my_lyear) {
                        $tmp_a='';
                        $tmp_changed='';
                        next;
                    }
                    else {
                        $my_cust_free += (Math::BigInt->new(Net::IP::ip_bintoint(Net::IP::ip_iptobin($tmp_b,4)))-
                             Math::BigInt->new(Net::IP::ip_bintoint(Net::IP::ip_iptobin($tmp_a,4)))+1);

                        if ($my_debug) {
                            print "DEBUG_MSG: [assignment_check_objectsize] sum: $my_cust_free | $tmp_a - $tmp_b\n";
                        }
                    }
                }
            } # end: for (@my_cust_query)

            if ($tmp_c == 0) {
                push @my_warnings, "AS-WINDOW: sorry, there are no available inetnums for netname $my_cust";
            }
            else {
            # check for assignment
                $tmp_whois->Query("-s $SOURCE -r -T range $tmp_firstinet");
                my $my_reply = $tmp_whois->GetResult();
                if ($my_reply =~ /registry:\s+(\S+)/s) {
                    $tmp_whois->Query("-s $SOURCE -T reg-id $1");
                    my $my_reply = $tmp_whois->GetResult();
                    if ($my_reply =~ /window:\s+(\d+)/s) {
                        my $calc_win = $1;
                        $calc_win = 2 ** (32-$calc_win);
                        if ($calc_win <= $my_cust_free) {
                            push @my_warnings, "AS-WINDOW-WARNING: limit exceeded ($my_cust_free / $calc_win)";
                        }
                        if ($my_debug) {
                            print "DEBUG_MSG: [assignment_check_stat] $my_cust_free / $1 (free/limit)\n";
                        }
                    }
                    else {
                        die "reg-id without as-window cannot be possible!";
                    }
                }
                else {
                    push @my_warnings, "AS-WINDOW: sorry, there are no available ranges for netname/inetnum $my_cust/$tmp_firstinet";
                }
            } # inetnums found!
        } # my_cust


# --> [QUERY_REQ]
# query ranges (-M range_start-range_end)

#######################
#
# START fr IP_REQUEST
#
#######################

        for (@ip_request) {
            ($tmp_range_start,$tmp_range_end,$tmp_pref,$tmp_purpose,$tmp_location,$tmp_used,$tmp_netname) = split(/\|/,$_);

            # --> inetnum objects in -r -M query
        	if ($tmp_used) {
        	    my $dummy_start  = Net::IP::ip_iptobin($tmp_range_start,4);
                my $dummy_end    = Net::IP::ip_iptobin($tmp_range_end,4);
                $tmp_range_start = Math::BigInt->new(Net::IP::ip_bintoint($dummy_start));
                $tmp_range_end   = Math::BigInt->new(Net::IP::ip_bintoint($dummy_end));
                $my_used         = $tmp_range_end-$tmp_range_start+1;
                push @ip_data, "$count_real_ranges|$dummy_start|$dummy_end|1|$my_used|$tmp_netname|I";
                if ($my_debug) {
                    print "DEBUG_MSG: [IP_USED] $count_real_ranges|$dummy_start|$dummy_end|1|$my_used|$tmp_netname|I\n";
                }
                next;
            }

            if ($my_debug) {
                print "DEBUG_MSG: [QUERY_RANGE] $tmp_range_start - $tmp_range_end (\$my_range: $my_range | \$my_regid: $my_regid | \$my_list: $my_list) [dump to debug.txt]\n";
            }

            if (($my_range ne '') or ($my_regid ne '') or ($my_list ne '') or ($my_purpose ne '') or ($my_location ne '')) {
                $tmp_whois->Query("-s $SOURCE -r -T range $tmp_range_start-$tmp_range_end");
                my @tmp_query_b=split(/\n/,$tmp_whois->GetResult());
                $tmp_query=join("\n",@tmp_query_b);

                $tmp_whois->Query("-s $SOURCE -r -T inetnum $tmp_range_start-$tmp_range_end");
                my @tmp_query_b=split(/\n/,$tmp_whois->GetResult());
                $tmp_query.="\n".join("\n",@tmp_query_b);

                $tmp_whois->Query("-s $SOURCE -r -M $tmp_range_start-$tmp_range_end");
                my @tmp_query_a = split(/\n/,$tmp_whois->GetResult());
                $tmp_query.="\n".join("\n",@tmp_query_a);

                $tmp_query =~ s/^%.*$//gm;
                $tmp_query =~ s/(\n\n)\n+/$1/gs;

                if ($my_debug) {
                    open(DEBUG,">>debug.txt");
                    print DEBUG "[A] -r -T range $tmp_range_start-$tmp_range_end | -r -M $tmp_range_start-$tmp_range_end\n";
                    print DEBUG $tmp_query . "\n[EOF]\n";
                    close(DEBUG);
                }

                }
                else {
                    $tmp_whois->Query("-s $SOURCE -r -M $tmp_range_start-$tmp_range_end");
                    $tmp_query=$tmp_whois->GetResult();

                    if ($my_debug) {
                        open(DEBUG,">>debug.txt");
                        print DEBUG "[B] -r -M $tmp_range_start-$tmp_range_end\n";
                        print DEBUG $tmp_query . "\n[EOF]\n";
                        close(DEBUG);
                    }
                }

            if (($my_range eq '') and ($my_regid eq '') and ($my_list eq '')) {
                @tmp_ip_data=&read_out_inet($tmp_query,$count_real_ranges);
            } # Getting details for range
            else {
                @tmp_ip_data=&read_out($tmp_query,1);
            }

            my ($scan_area_start,$scan_area_end,$end,$marker,$bin_start, $bin_end, $last_used_start, $last_used_end,$my_used,$last_netnam,$last_netname);

            $scan_area_start = Net::IP::ip_iptobin($tmp_range_start,4);
            $scan_area_end   = Net::IP::ip_iptobin($tmp_range_end,4);

            $end             = Math::BigInt->new(Net::IP::ip_bintoint($scan_area_end));
            $marker          = Math::BigInt->new(Net::IP::ip_bintoint($scan_area_start));

            # get range-options // for any ranges in non -cl mode i need to modify get_inet-objects // saving range_objects
            # so first only std usage!

            # Creating a list of available IP-Ranges
            my @cache_tmp_ip_data = @tmp_ip_data; @tmp_ip_data = ();

            for (@cache_tmp_ip_data) {
                ($x,$last_used_start,$last_used_end,$last_netname,$tmp_purpose,$tmp_location,$my_used) = split(/\|/,$_);

                $r_last_used_start = Net::IP::ip_iptobin($last_used_start,4);
                $r_last_used_end   = Net::IP::ip_iptobin($last_used_end,4);

                if ($my_used==0) {
                    if ($my_debug) {
                        print "DEBUG_MSG: [Rangetable] $last_used_start|$last_used_end|$tmp_purpose|$tmp_location\n";
                    }
                    push @my_ranges, "$r_last_used_start|$r_last_used_end|$tmp_purpose|$tmp_location";
                    push @tmp_ip_data, "0|$last_used_start|$last_used_end|CLASS|$tmp_purpose|$tmp_location|1" if ( ($my_regid ne '') or ($my_range ne '')); # TEMP_FIX
                }
            }

            for (@cache_tmp_ip_data) {
                ($x,$last_used_start,$last_used_end,$last_netname,$tmp_purpose,$tmp_location,$my_used) = split(/\|/,$_);

                $r_last_used_start = Net::IP::ip_iptobin($last_used_start,4);
                $r_last_used_end   = Net::IP::ip_iptobin($last_used_end,4);

                if ($my_used==1) {
                    ($tmp_purpose,$tmp_location,undef) = check_ranges($r_last_used_start,$r_last_used_end,$tmp_purpose,$tmp_location);
                    push @tmp_ip_data, "$x|$last_used_start|$last_used_end|$last_netname|$tmp_purpose|$tmp_location|$my_used";
                }
            }

#            # Protection for too big ranges
#            my $tmp_last_used_start = Math::BigInt->new(Net::IP::ip_bintoint($r_last_used_start));
#            my $tmp_last_used_end   = Math::BigInt->new(Net::IP::ip_bintoint($r_last_used_end));
#            
#            if ($tmp_last_used_start < $tmp_range_start) {
#                if ($my_debug) {
#                    print "DEBUG_MSG: [IS_TOO_SMALL] correction ($tmp_last_used_start < $tmp_range_start)\n";
#                }
#                $last_used_start=Net::IP::ip_bintoip($my_range_start_b,4);
#            }
#            if ($tmp_last_used_end > $tmp_range_end) {
#                if ($my_debug) {
#                    print "DEBUG_MSG: [IS_TOO_BIG] correction ($tmp_last_used_end > $tmp_range_end)\n";
#                }
#                $last_used_end=Net::IP::ip_bintoip($my_range_stop_b,4);
#            }

            &ip_sort(\@tmp_ip_data,1) if ( ($my_regid ne '') or ($my_range ne '')); # TEMP_FIX

            # creating IP_USE_TABLE
            for (@tmp_ip_data) {
                (undef,$last_used_start,$last_used_end,$last_netname,$tmp_purpose,$tmp_location,$my_used) = split(/\|/,$_);
                my $my_extopt=0;
                if ($tmp_purpose =~ /^reclaim$/i) {
                    $my_extopt='1';
                }

                if ($my_debug) {
                    print "DEBUG_USED_SPACE: $last_used_start - $last_used_end [$last_netname|$tmp_purpose|$tmp_location|$my_used]\n";
                } # DEBUG

                $last_used_start = Math::BigInt->new(Net::IP::ip_bintoint(Net::IP::ip_iptobin($last_used_start,4)));
                $last_used_end   = Math::BigInt->new(Net::IP::ip_bintoint(Net::IP::ip_iptobin($last_used_end,4)));

                if ($last_used_start==$marker) {
                    #--> used ip-space
                    $my_used=$last_used_end-$last_used_start+1;
                    $bin_start=$last_used_start->as_bin();     $bin_start =~ s/^..//;
                    $bin_end=$last_used_end->as_bin();     $bin_end =~ s/^..//;
                    push @ip_data, "$count_real_ranges|$bin_start|$bin_end|1|$my_used|$last_netname|R|$my_extopt";
                    $marker=$last_used_end->copy();
                    $marker->binc();
                }
                elsif ($last_used_start>$marker) {
                    #--> unused ip-space
                    $my_used=$last_used_start-$marker;
                    $bin_start=$marker->as_bin();
                    $bin_start =~ s/^..//;
                    $bin_end=$last_used_start->copy;
                    $bin_end->bdec();
                    $bin_end=$bin_end->as_bin();
                    $bin_end =~ s/^..//;
                    push @ip_data, "$count_real_ranges|$bin_start|$bin_end|0|$my_used||R|";
                    $marker=$last_used_start->copy();

                    #--> used ip-space
                    $my_used=$last_used_end-$last_used_start+1;
                    $bin_start=$last_used_start->as_bin();
                    $bin_start =~ s/^..//;
                    $bin_end=$last_used_end->as_bin();
                    $bin_end =~ s/^..//;
                    push @ip_data, "$count_real_ranges|$bin_start|$bin_end|1|$my_used|$last_netname|R|$my_extopt";
                    $marker=$last_used_end->copy();
                    $marker->binc();
                }
            } # for (@tmp_ip_data)

            # something forgotten? if marker not reached $scan_area_end[int]
            $scan_area_end = Math::BigInt->new(Net::IP::ip_bintoint($scan_area_end,4));

            if ($scan_area_end>$marker) {
                #--> unused ip-space
                $my_used=$scan_area_end-$marker+1;
                $bin_start=$marker->as_bin();
                $bin_start =~ s/^..//;
                $bin_end=$scan_area_end->as_bin();
                $bin_end =~ s/^..//;;
                push @ip_data, "$count_real_ranges|$bin_start|$bin_end|0|$my_used||R|";
            }

            # @ip_data
            # FORMAT: <range-id>|<IP-Start>|<IP-End>|<Status>|<size>|<netname>|<opt_1>|<opt_2>
            # Status: 0 -> FREE | 1 -> used
            # opt_1: R(ange) /
            # opt_2 1: is_reclaim

            $count_real_ranges++;
        } # ip_request


#######################
#
# ENDE fr IP_REQUEST
#
#######################

        &message($Conf::msg[11],9) unless @ip_data; # for debugging purposes - it may happen!

        # LOOKING FOR BEST-IP-SPACE
        my ($ip_range,$ip_start, $ip_end, $ip_status, $ip, $ip_prefix, $ip_used,$ip_netname, $mesg, @prefix, $my_stat_used, $my_stat_unused, @prn_prefix, %statistics, $ip_opt, $ip_eopt);
        my $ip_range_last=-1;
        my $count=0;
        my $first=1;
        my $has_found = 0;
        my $stat_ip_total=0;

        for (@ip_data) {
            ($ip_range, $ip_start, $ip_end, $ip_status, $ip_used, $ip_netname, $ip_opt, $ip_eopt) = split(/\|/,$_);
            $ip_start = "00".$ip_start; $ip_start =~ s/.*(.{32})$/$1/;
            $ip_end   = "00".$ip_end;   $ip_end   =~ s/.*(.{32})$/$1/;
            $ip_eopt  = 0 if ($ip_eopt eq '');

            if ($my_debug) {
                my $a = Net::IP::ip_bintoip($ip_start,4);
                my $b = Net::IP::ip_bintoip($ip_end,4);
                print "DEBUG_MSG: [ip_data] $ip_range|$a|$b|$ip_status|$ip_used|$ip_netname\n"
            }

#   #        if ($ip_opt eq 'R')
            {
                if ($ip_status==1) {
                    $statistics{$ip_range}{'used'}+=$ip_used;
                    $statistics{$ip_range}{'rused'}+=$ip_used;
                    $statistics{'all'}{'used'}+=$ip_used;
                    $statistics{'all'}{'rused'}+=$ip_used if ($ip_opt eq 'R');
                }
                else {
                    $statistics{$ip_range}{'free'}+=$ip_used;
                    $statistics{$ip_range}{'rfree'}+=$ip_used;
                    $statistics{'all'}{'free'}+=$ip_used;
                    $statistics{'all'}{'rfree'}+=$ip_used if ($ip_opt eq 'R');
                }
            }
        }

        if ($my_back) {
            @ip_data = reverse @ip_data;
        }

        my ($tmp_count, $tmp_rcount, $tmp_free, $tmp_free_per, $tmp_used, $tmp_used_per, $cnt_range);
        $tmp_count=$statistics{'all'}{'free'}+$statistics{'all'}{'used'};
        $tmp_rcount=$statistics{'all'}{'rfree'}+$statistics{'all'}{'rused'};
        $tmp_free_per=roundme($statistics{'all'}{'free'}/$tmp_count*100);
        $tmp_used_per=roundme($statistics{'all'}{'used'}/$tmp_count*100);

        if ($statistics{'all'}{'rfree'} eq '') {
            $statistics{'all'}{'rfree'}=0;
        }
        if ($statistics{'all'}{'rused'} eq '') {
            $statistics{'all'}{'rused'}=0;
        }

# --> [REQUEST]
        REQUEST:
        # statistics: counting free/used ip-space
        my $x=0;      # counter
        $cnt_range=0; # counter for range

        # show header
        print<<END if ( ($first) && ($my_short==2) );
Range                                Used    Free
-------------------------------------------------------------------------------
END

        if ( ($first) && ($my_short==3) ) {
            print "Range Statistics\n\n";
            print "Purpose     : $tmp_purpose\n" if $tmp_purpose;
            print "Location    : $tmp_location\n" if $tmp_location;

            formline <<END, $count_real_ranges, $tmp_rcount, $statistics{'all'}{'rfree'}, $tmp_free_per, $statistics{'all'}{'rused'}, $tmp_used_per;
Ranges      : @>>>>>
Total       : @>>>>>
Free        : @>>>>> (@>>%)
Used        : @>>>>> (@>>%)

Range                                Total          Free          Used      Preference
---------------------------------------------------------------------------------------
END
            print $^A; $^A='';
        }

# show data
##############################################################################################################

        if ($my_debug==1) {
            print "DEBUG_MSG: \$my_cmd_sub=$my_cmd_sub | \$my_short=$my_short | \$my_templ_save=$my_templ_save | \$ip_status=$ip_status\n";
        }

        my @ip_data_tmp;
        for (@ip_data) {
            ($ip_range, $ip_start, $ip_end, $ip_status, $ip_used,$ip_netname,$ip_opt,$ip_eopt) = split(/\|/,$_);

            $ip_start = "00".$ip_start; $ip_start =~ s/.*(.{32})$/$1/;
            $ip_end   = "00".$ip_end;   $ip_end   =~ s/.*(.{32})$/$1/;

            my $tmp_out = Net::IP::ip_bintoip($ip_start,4).' - '.Net::IP::ip_bintoip($ip_end,4);

            if ($ip_range != $ip_range_last) {
                if ($my_debug) {
                    print "DEBUG_MSG: [new_range] last: $ip_range_last | now: $ip_range\n";
                }
                $cnt_range=0;
                $x++;
                ($tmp_range_start,$tmp_range_end,$tmp_pref,$tmp_purpose,$tmp_location) = split(/\|/,$ip_request[$ip_range]);

                if (!defined($statistics{$ip_range}{'used'})) {
                    $statistics{$ip_range}{'used'}=0;
                }
                if (!defined($statistics{$ip_range}{'free'})) {
                    $statistics{$ip_range}{'free'}=0;
                }
                $tmp_count=$statistics{$ip_range}{'free'}+$statistics{$ip_range}{'used'};
                $tmp_free_per=roundme($statistics{$ip_range}{'free'}/$tmp_count*100);
                $tmp_used_per=roundme($statistics{$ip_range}{'used'}/$tmp_count*100);

                if ($ip_opt eq 'I') {
                    push @ip_data_tmp, "$tmp_range_start|$tmp_range_end|$ip_used|$ip_used|$tmp_pref";
                }

                if ( $my_templ_save<2 ) {
                    if ($my_short==2) {
                        formline <<END, $tmp_range_start, $tmp_range_end, $tmp_used_per, $tmp_free_per;
@<<<<<<<<<<<<<< - @<<<<<<<<<<<<<<   @>>>%   @>>>%
END
                        print $^A; $^A='';
                    }

                    if ( ($my_short==1) && ($first) ) {
                        print <<END;
---------------------------------------------------------------------------------
Address range                         total  cidr  Network
---------------------------------------------------------------------------------
END
                    }
                }
                $first = 0 if $first;
                $ip_range_last=$ip_range;
            } # every range

            # --> used_ip_space
            if ( (($my_cmd_sub==1) or ($my_cmd_sub==2)) && ($ip_status==1)) {
                if ($my_templ_save<2) {
                    @prn_prefix = Net::IP::ip_range_to_prefix($ip_start, $ip_end, 4);
                    (undef,$prn_prefix[0]) = split(/\//,$prn_prefix[0]);

                    my $tmp_place = $ip_netname;
                    if (($ip_eopt == 1) && ($my_list eq '')) {
                        $tmp_place = "* * RECLAIM * *";
                    }

                    formline <<END, $tmp_out,$ip_used, "/$prn_prefix[0]",$tmp_place;
@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<   @>>>>>   @>>   @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
END
                    print $^A; $^A='';
                } $tmp_out=''; $cnt_range++;
            }

            # --> free_ip_space
            elsif ( ($ip_status == 0) && (($my_cmd_sub == 0) or ($my_cmd_sub == 2)) ) {
                my $ip_start_int = Math::BigInt->new(Net::IP::ip_bintoint($ip_start));
                my $ip_end_int   = Math::BigInt->new(Net::IP::ip_bintoint($ip_end));

                $ip_start = Net::IP::ip_inttobin($ip_start_int,4);
                @prefix = Net::IP::ip_range_to_prefix($ip_start, $ip_end, 4);

                for (@prefix) {
                    if ( ($my_show != -1) && ($count>=$my_show) ) {
                        last;
                    }
                    if ($_ =~ /([^\/]+)\/(\d+)/) {
                        $ip = $1; $ip_prefix=$2;
                        if ( ($my_sizeincidr) && ($my_sizeincidr != $ip_prefix) ) {
                            next;
                        }

                        my ($tmp_a,$tmp_b,$tmp_a_b,$tmp_b_b);

                        if ($my_sizeincidr_org ne '') {
                            ($tmp_a,$tmp_b) = Net::IP::ip_prefix_to_range($ip,$my_sizeincidr_org,4);
                            $ip_prefix=$my_sizeincidr_org;
                        }
                        else {
                            ($tmp_a,$tmp_b) = Net::IP::ip_prefix_to_range($ip,$ip_prefix,4);
                        }

                        # something found
                        $has_found=1;

                        if ($my_templ_save>0) {
                            # insert template
                            @tmp_template_copy=@tmp_template;
                            for (@tmp_template_copy) {
                                $_ =~ s/<netname>/$my_templ_netname/;
                                $_ =~ s/<netrange>/$tmp_a - $tmp_b/;
                                if ($my_templ_save==1) {
                                    print TEMPL $_ if $my_templ_save;
                                }
                                else {
                                    print $_;
                                }
                            }
                        }
                        if ( ($my_templ_save<2) && ($my_short == 1) ) {
                            $cnt_range++;
                            $tmp_out = "$tmp_a - $tmp_b";

                            $tmp_a_b = Net::IP::ip_iptobin($tmp_a,4);
                            $tmp_b_b = Net::IP::ip_iptobin($tmp_b,4);

                            $tmp_a = Math::BigInt->new(Net::IP::ip_bintoint($tmp_a_b));
                            $tmp_b = Math::BigInt->new(Net::IP::ip_bintoint($tmp_b_b));

                            $ip_start_int = $tmp_b+1;

                            $ip_used = $tmp_b-$tmp_a+1;

                            my $is_free;
                            ($my_purpose,$my_location,$is_free) = check_ranges($tmp_a_b,$tmp_b_b,$my_purpose,$my_location);

                            if ($my_purpose =~ /^reclaim$/i) {
                                $mesg = "* * RECLAIM * *";
                            }
                            elsif (($is_free) && (($my_regid ne '') or ($my_range ne ''))) {
                                $mesg = "* * NO_CLASS * *";
                            }
                            else {
                                $mesg = "* * FREE * *"; next if ( ($my_regid ne '') or ($my_range ne ''));
                            }

                            formline <<END, $tmp_out,$ip_used,"/$ip_prefix",$mesg;
@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<   @>>>>>   @<<   @<<<<<<<<<<<<<<<<<<<
END
                            print $^A; $^A='';
                        }
                        $tmp_out='';
                    }
                    $count++;
                    if ($my_free_all==0) {
                        last;
                    }
                } # for (@prefix)
                if ( ($my_show != -1) && ($count>=$my_show) ) {
                    last;
                }
            }
        } # every IP_ENTRY

        # range_statistics:
        if ($my_short==3) {
            $x = 0;
            for (@ip_request) {
                ($tmp_range_start,$tmp_range_end,$tmp_pref,$tmp_purpose,$tmp_location,$tmp_used) = split(/\|/,$_);
                next if $tmp_used;
                $tmp_free_per=roundme($statistics{$x}{'free'}/$tmp_count*100);
                $tmp_used_per=roundme($statistics{$x}{'used'}/$tmp_count*100);
                formline <<END, $tmp_range_start, $tmp_range_end, $tmp_count, $statistics{$x}{'free'}, $tmp_free_per, $statistics{$x}{'used'}, $tmp_used_per, $tmp_pref;
@<<<<<<<<<<<<<< - @<<<<<<<<<<<<<<   @>>>>>  @>>>> (@>>%)  @>>>> (@>>%)  @>>>>
END
                print $^A; $^A='';
                $x++;
            }
        }
        # end of: range_statistics

        if ( $my_templ_save<2 ) {
            print <<END;
---------------------------------------------------------------------------------
END
        }

        if (($my_short==3) && (@ip_data_tmp!=0)) {
            print $Conf::msg[12]; print "\n";
            my $tmp_ip_count=0;
            for (@ip_data_tmp) {
                ($tmp_range_start, $tmp_range_end, $tmp_count, $tmp_used, $tmp_pref) = split(/\|/,$_);
                $tmp_ip_count += $tmp_count;
                formline <<END, $tmp_range_start, $tmp_range_end, $tmp_count, "0", "100", $tmp_used, "100", $tmp_pref if ($ip_opt eq 'R');
@<<<<<<<<<<<<<< - @<<<<<<<<<<<<<<   @>>>>>  @>>>> (@>>%)  @>>>> (@>>%)  @>>>>
END
                print $^A; $^A='';
            }
            print <<END;
---------------------------------------------------------------------------------
Total inetnum: $tmp_ip_count
END

        }

        if ($x>1) {
            if (($count==0) && ($cnt_range==0) && ($my_short!=3) and ($my_sizeincidr eq '')) {
                my $message = "$my_purpose ";
                if ($my_location) {
                    $message .= "and $my_location";
                }
                &message($Conf::msg[9],1,$message)
            }

            if ( (!$my_short) and ( $my_templ_save<2 ) ) { # still used?
                print <<END;
Total     : $tmp_count
Free    : $statistics{'all'}{'free'} ($tmp_free_per%)
Used    : $statistics{'all'}{'used'} ($tmp_used_per%)
END
            }
        }

        if ( ($has_found==0) and ($my_sizeincidr ne '') and ($my_sizeincidr>12) ) {
            $my_sizeincidr--;
            if ($my_debug==1) {
                print "DEBUG_MSG: [cidr_search] switching to $my_sizeincidr (org: $my_sizeincidr_org)\n";
            }
            goto REQUEST;
        }
        if ( (!$first) && ($has_found==0) && ($my_sizeincidr ne '') ) {
            &message($Conf::msg[8],1,"$tmp_purpose/$tmp_location [$my_sizeincidr]");
        }
    }
    else {
        die "Unknown Command!";
    }

    if ( $my_templ_save==1 ) {
        close(TEMPL);
    }
    for (@my_warnings) { # New / 07/2003
        print "Warning: $_\n";
    }
}

##############################################################

##
## round (statistics)
##

sub roundme {
    my $back;
    if ($_[0] =~ /^(\d+\.\d)(\d)/) {
        if ($2 > 4) {
            $back = $1+0.1;
        }
        else {
            $back = $1;
        }
    }
    else {
        $back = $_[0];
    }
    return $back;
}

##############################################################

##
## parsing out ranges|inetnums
##

sub read_out {
    my ($my_data,$opt) = @_;

    if ($my_debug) {
        print "DEBUG_MSG: sub read_out(query,$opt)\n";
    }

    my @my_data = split(/\n/,$my_data);
    push @my_data, '';
    my ($tmp_range_start, $tmp_range_end, $tmp_pref, $tmp_purpose, $tmp_location, $my_used, $tmp_netname, $line, $tmp_status);
    my $scan=0;
    @ip_data_lp = () if $opt;

    foreach $line (@my_data) {
        if ($scan==0) {
            if ($line =~ /^range:(\s|\t)*(\d+\.\d+\.\d+\.\d+) \- (\d+\.\d+\.\d+\.\d+)(\s|\t)*$/) {
                $tmp_range_start=$2;
                $tmp_range_end=$3;
                $tmp_pref=$Conf::RANGE_PREF;
                $scan=1;
                $tmp_purpose='';
                $tmp_location='';
                $my_used=0;
                $tmp_netname='';
            }
            if ($line =~ /^inetnum:(\s|\t)*(\d+\.\d+\.\d+\.\d+) \- (\d+\.\d+\.\d+\.\d+)(\s|\t)*$/) {
                {
                    $tmp_range_start=$2;
                    $tmp_range_end=$3;
                    $tmp_pref=0;
                    $scan=1;
                    $tmp_purpose='';
                    $tmp_location='';
                    $my_used=1;
                }
            }
        }
        elsif ($scan==1) {
            if ($line =~ /^preference:\s*(\d+)/) {
                $tmp_pref    = $1;
            }
            elsif ($line =~ /^usedfor:\s*(.+)$/) {
                $tmp_purpose = $1
            }
            elsif ($line =~ /^location:\s*(.+)$/) {
                $tmp_location     = $1;
            }
            elsif ($line =~ /^netname:\s*(.+)$/) {
                $tmp_netname = $1;
            }
            elsif ($line =~ /^status:\s*(.+)$/) {
                $tmp_status    = $1;
            }
            elsif ($line eq '') {
                $scan=0;

                if ( ($my_used !=1) && ($tmp_status !~ /^allocated .*$/i) ) {

                    if ($opt) {
                        push @ip_data_lp, "1|$tmp_range_start|$tmp_range_end|$tmp_netname|$tmp_purpose|$tmp_location|$my_used";
                    }
                    else {
                        push @ip_data_lp, "$tmp_range_start|$tmp_range_end|$tmp_pref|$tmp_purpose|$tmp_location|$my_used|$tmp_netname";
                    }
                    if ($my_debug) {
                        print "DEBUG_MSG: [read_out-$opt] $tmp_range_start|$tmp_range_end|$tmp_pref|$tmp_purpose|$tmp_location|$my_used|$tmp_netname\n";
                    }
                }
                $tmp_purpose='';
                $tmp_location='';
                $tmp_netname='';
                $tmp_status='';

            } # empty line => new object
        } # elsif ($scan==1)
    } # each line

    # ORDER BY IP
    # --> order by IP
    my ($order_a_1,$order_a_2,$order_a_3,$order_a_4,$order_b_1,$order_b_2,$order_b_3,$order_b_4,$tmp_a_ip,$tmp_b_ip);

    @ip_data_lp = sort {
        ($tmp_a_ip,undef) = split(/\|/,$a,2) unless $opt;
        ($tmp_b_ip,undef) = split(/\|/,$b,2) unless $opt;
        (undef,$tmp_a_ip,undef) = split(/\|/,$a,3) if $opt;
        (undef,$tmp_b_ip,undef) = split(/\|/,$b,3) if $opt;

        $tmp_a_ip =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/;$order_a_1=$1; $order_a_2=$2; $order_a_3=$3; $order_a_4=$4;
        $tmp_b_ip =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/; $order_b_1=$1; $order_b_2=$2; $order_b_3=$3; $order_b_4=$4;

        $order_b_1 <=> $order_a_1 || $order_b_2 <=> $order_a_2 || $order_b_3 <=> $order_a_3 || $order_b_4 <=> $order_a_4
    } @ip_data_lp;

    return reverse @ip_data_lp if $opt;
}

##############################################################

##
## parsing out inetnum-objects & order them [reverse if needed]
##

sub read_out_inet {
    my ($my_data,$my_range,$my_opt) = @_;

    if ($my_debug) {
        print "DEBUG_MSG: sub read_out_inet(query,$my_range,$my_opt)\n";
    }

    my @my_data = split(/\n/,$my_data);
    push @my_data, '';
    my ($tmp_range_start, $tmp_range_end, $tmp_netname,$tmp_purpose,$tmp_location,$tmp_status, $line, @ip_data_unsorted);
    my $scan=0;

    foreach $line (@my_data) {
        if ($scan==0) {
            if ($line =~ /^inetnum:(\s|\t)*(\d+\.\d+\.\d+\.\d+) \- (\d+\.\d+\.\d+\.\d+)(\s|\t)*$/) {
                $tmp_range_start=$2;
                $tmp_range_end=$3;
                $scan=1;
                $tmp_netname='';
                $tmp_purpose='';
                $tmp_location='';
            }
        }
        elsif ($scan==1) {
            if ($line =~ /^netname:\s*(.+)$/) {
                $tmp_netname=$1;
            }
            elsif ($line =~ /^usedfor:\s*(.*)/) {
                $tmp_purpose=$1;
            }
            elsif ($line =~ /^location:\s*(.*)/) {
                $tmp_location=$1;
            }
            elsif ($line =~ /^status:\s*(.*)/) {
                $tmp_status=$1;
            }
            if ($line eq '') {
                if (($my_opt!=1) && ($tmp_status !~ /^allocated .*$/i)) {
                    push @ip_data_unsorted, "$my_range|$tmp_range_start|$tmp_range_end|$tmp_netname|$tmp_purpose|$tmp_location|1";
                }
                $scan=0;
            }
        } # elsif ($scan==1)
    }

    # ORDER BY IP
    # --> order by IP

    my $order_a_1;
    my $order_a_2;
    my $order_a_3;
    my $order_a_4;
    my $order_b_1;
    my $order_b_2;
    my $order_b_3;
    my $order_b_4;
    my $tmp_a_ip;
    my $tmp_b_ip;

    @ip_data_unsorted = reverse sort {
        (undef,$tmp_a_ip,undef) = split(/\|/,$a,3);
        (undef,$tmp_b_ip,undef) = split(/\|/,$b,3);

        $tmp_a_ip =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/; $order_a_1=$1; $order_a_2=$2; $order_a_3=$3; $order_a_4=$4;
        $tmp_b_ip =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/; $order_b_1=$1; $order_b_2=$2; $order_b_3=$3; $order_b_4=$4;

        $order_b_1 <=> $order_a_1 || $order_b_2 <=> $order_a_2 || $order_b_3 <=> $order_a_3 || $order_b_4 <=> $order_a_4
    } @ip_data_unsorted;

    return @ip_data_unsorted; # sorted!
}

##############################################################

##
## sql_query_load(<what>); 1=purposes | 2=locations
##

sub sql_query_load {
    my @tmp_dbdata;
    my $des;

    if ($_[0]==1) {
        my $tmp_sth = $tmp_dbh->prepare("SELECT purpose,object FROM purpose,last WHERE purpose.object_id=last.object_id and purpose.thread_id = 0 ORDER BY purpose");
        my $tmp_rc = $tmp_sth->execute || &message($Conf::msg[3],9,DBI->errstr);
        while (@tmp_dbdata = $tmp_sth->fetchrow_array) {
            push @glb_purpose, uc($tmp_dbdata[0]);
            if ($tmp_dbdata[1] =~ /descr:[\s\t]+([^\n]+)/) {
                $des=$1;
            }
            push @glb_purpose_ext, uc($tmp_dbdata[0])."|$des";
        }
    }
    elsif ($_[0]==2) {
        my $tmp_sth = $tmp_dbh->prepare("SELECT pop,aut_num,object FROM pop,as_ref,aut_num,last WHERE pop.object_id=last.object_id and aut_num.object_id=as_id and as_ref.object_id=pop.object_id and pop.thread_id = 0 ORDER BY pop");

        my @tmp_rc = $tmp_sth->execute || &message($Conf::msg[3],9,DBI->errstr);
        while (@tmp_dbdata = $tmp_sth->fetchrow_array) {
            push @glb_location, uc($tmp_dbdata[0]);
            if ($tmp_dbdata[2] =~ /address:[\s\t]+([^\n]+)/) {
                $des=$1;
            }
            push @glb_location_ext, uc($tmp_dbdata[0])."|$tmp_dbdata[1]|$des";
        }
    }
}

##############################################################

##
## message('<message>',<status>);
##

sub message {

# Status:
# 0 ->
# 9 -> warnings, exit

    my ($my_msg,$my_status,$my_insert) = @_;
    my ($tmp_sub,$tmp_file,$tmp_line) = caller;

    $my_msg =~ s/\#msg/$my_insert/g;

    my ($t_sec, $t_min, $t_std, $t_tag, $t_mon, $t_jahr, undef, undef, undef) = localtime(time);
    $t_mon++;
    if ($t_mon == 13) {
        $t_mon = 1;
    }
    $t_jahr += 1900;
    for ($t_sec, $t_min, $t_std, $t_tag, $t_mon) {
        $_ =~ s/^(\d)$/0$1/;
    }

    my $date = "$t_jahr-$t_mon-$t_tag $t_std:$t_min:$t_sec";
    $my_msg .= "$date $my_msg [$tmp_line]\n";

    if ($Conf::file{'log'} ne '') {
        open(LOG,">>$Conf::file{'log'}");
        print LOG $my_msg;
        close(LOG);
    }
    else {
        print $my_msg;
    }
    if ($my_status>8) {
        exit;
    }
}

##############################################################

##
## show_purposes
##

sub show_purposes {
    &sql_query_load(1);

    print <<END;
-------------------------------------------------------------
Purpose                          Description
-------------------------------------------------------------
END

    my @dummy;

    for (@glb_purpose_ext) {
        @dummy = split(/\|/,$_);

        formline <<END, @dummy;
@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
END
        print $^A; $^A='';
    }

    print "-------------------------------------------------------------\n";
    exit;
}

##############################################################

##
## show_facility
##

sub show_facility {
    &sql_query_load(2);

    print <<END;
------------------------------------------------------------------
Facility                           AS-No             Address
------------------------------------------------------------------
END

    my @dummy;

    for (@glb_location_ext) {
        @dummy = split(/\|/,$_);

        formline <<END, @dummy;
@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<   @<<<<<<<<<<<<<<   @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
END
        print $^A; $^A='';
    }

    print "------------------------------------------------------------------\n";

    for (@my_warnings) {
        print "Warning: $_\n";
    }
    exit;
}

##############################################################

##
## IP_SORT_ENGINE
##
## 1. param: address of array to be sorted | 2. param: position of ip-address
## source ip-space: @ip_data_lp

sub ip_sort {
    my ($ip_data, $pos) = @_;

    # --> order by IP
    my ($order_a_1,$order_a_2,$order_a_3,$order_a_4,$order_b_1,$order_b_2,$order_b_3,$order_b_4,$tmp_a_ip,$tmp_b_ip);

    @$ip_data = reverse sort {
        my @array_a = split(/\|/,$a,$pos+1); $tmp_a_ip = $array_a[$pos];
        my @array_b = split(/\|/,$b,$pos+1); $tmp_b_ip = $array_b[$pos];

        $tmp_a_ip =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/; $order_a_1=$1; $order_a_2=$2; $order_a_3=$3; $order_a_4=$4;
        $tmp_b_ip =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/; $order_b_1=$1; $order_b_2=$2; $order_b_3=$3; $order_b_4=$4;

        $order_b_1 <=> $order_a_1 || $order_b_2 <=> $order_a_2 || $order_b_3 <=> $order_a_3 || $order_b_4 <=> $order_a_4
    }
    @ip_data_lp;
}

##############################################################

##
## Check if IP is within the ranges
##
## uses @my_ranges

sub check_ranges {
    my ($my_bip_from,$my_bip_to,$my_pur,$my_loc) = @_;
    for (@my_ranges) {
        my ($r_start,$r_end,$r_pur,$r_loc) = split(/\|/,$_);
        my $r_status = Net::IP::ip_is_overlap($r_start,$r_end,$my_bip_from,$my_bip_to,4);
        if (($r_status == $Net::IP::IP_IDENTICAL) or ($r_status == $Net::IP::IP_B_IN_A_OVERLAP)) {
            $my_pur = $r_pur if ($my_pur eq '');
            $my_loc = $r_loc if ($my_loc eq '');
            return ($r_pur,$r_loc,0);
        }
    } # (@my_ranges)

    return ($my_pur,$my_loc,1);
}

##############################################################

##
## USAGE
##

sub usage {
    print <<END;

Version: $script_vers [$script_date]

aisst [-p <purpose>] [-f <facility>] [-size <sizeincidr>] -free [num]
      -assigned -list
      [-t netname] [-purposes] [-facilities]

where

  <purpose>       is an valid purpose-object                  (Required)
  <facility>      is an valid facility-object                 (Optional)
  <sizeincidr>    size of network in CIDR notation ex. "/27"

  You must specify at lease one <purpose> or one <location>.

  Per default "aisst" displays the first matching free ip-space.
  If an inetnum template in the users homedir exists (~/.inetnum)
  an according inetnum objekt is printed on stdout.

Options

-p <purpose>          Selects ip-space tagged with <purpose>
-f <facility>         Selects ip-space tagged with <facility>
-free <num>           List all free ip-space or, if given,
                      the first <num> matching free ip-ranges
-assigned             List all assigned ip-space
-list                 List free and assigned ip-space
-purposes             List all available purposes
-facility             List all available locations/facilities
-config               Specify alternative config-file <config>
-t [netname]          create inetnum template
                      (default filename: $Conf::TEMPL_OUT)
-o <filename>         save inetnum to file (use -oo to append to file)
-s split              split result in subnets
-b                    find ip-space "backwards"
-rs                   range stats
-cl <range>||<reg-id> requesting unclassified-ranges
-n <Value>            checking assignment-window for customer/netname

END
    exit;
}

# END

