#!@bashpath@
#
##################################################################################
##										##
##   Copyright (c) 2000 - 2004 Arnd Vehling <av@nethead.de>			##
##   Copyright (c) 2003 - 2004 Cable & Wireless	<ripe-software.cw.net>		##
##   Copyright (c) 2003 - 2004 Tobias Cremer <ipadmin@de.cw.net>		##
##										##
##										##
##   This file is part of ripe-ipadmin.						##
##										##
##   ripe-ipadmin is free software; you can redistribute it and/or modify	##
##   it under the terms of the GNU General Public License as published by	##
##   the Free Software Foundation; either version 2 of the License, or		##
##   (at your option) any later version.					##
##										##
##   ripe-ipadmin is distributed in the hope that it will be useful,		##
##   but WITHOUT ANY WARRANTY; without even the implied warranty of		##
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		##
##   GNU General Public License for more details.				##
##										##
##   You should have received a copy of the GNU General Public License		##
##   along with ripe-ipadmin; if not, write to the Free Software		##
##   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	##
##										##
##    $Id: convert_handle.sh,v 1.8 2004/12/03 09:24:09 tcremer Exp $			##
##										##
##################################################################################







## Source config file
. @confdir@/convert_handle.conf


# Define function for errors

function error(){
echo -e "\n\n\tUSAGE: $0 HDL [DEST_SOURCE]\n\n\tYou must give one person handle and the destination source as a parameter to $0.\n\tIf the destination source is omitted, it is treated as $SOURCE.\n\tSee manpage for further details.\n\n"

exit 1

}

function wrongsource(){

echo -e "\n\n\tThe source you've provided as suffix of the hdl $1 is not allowed. Please check your config file.\n\tAllowed sources are: \n\n\t${ALLOW_SOURCES[@]}\n\n\n"

exit 1

}



## First check if all arguments are available

if [ "$#" -lt "1" -o "$#" -gt "2" ]; then

        error

fi


## Determine the Source source from the given hdl

SSOURCE="`echo $1 | sed 's/.*-//'`"


## First check if the source source is in a list of allowed sources
## Alternatively we could use the allowed sources from "whois -q sources"

# Get number of sources in array
snum=${#ALLOW_SOURCES[@]}

## Loop around and check if the source is correct
for((f=0; f < $snum; f++))
do

	if [ "$SSOURCE" = "${ALLOW_SOURCES[$f]}" ]; then

		ok=$f
	
	fi

done

# If its not in the array, display error message and exit

if [ "$ok" = '' ]; then

wrongsource

fi

## Set source to standard from the config if it is omitted
## else use the command line argument
## Make it uppercase as well
echo -n "Setting Source... "

if [ "$2" != '' ]; then 

	SOURCE="`echo $2 | awk '{ print toupper($1) }'`"

fi

echo $SOURCE


## To avoid confusion, make the person handle upper case

HDL="`echo $1 | awk '{ print toupper($1) }'`"

## Now get the object from the RIPE DB
echo -n "Retrieving object from ${ALLOW_HOSTS[$ok]}... "
whois -h ${ALLOW_HOSTS[$ok]} -p 43 -T person $HDL > /tmp/out.$$

## Ask for the status of the grep, i.e. look if the whois was successful
status=`grep "ERROR:101" /tmp/out.$$`

if [ "$?" == "0" ]; then 

	echo -e "\n\n\tThe person handle you are looking for\n\tdoes not exist in the ${ALLOW_SOURCES[$ok]} DB\n\n"; 
	cat /tmp/out.$$ | grep -v ^% | grep -v ^$

	exit 1

else

	echo -e "Ok"

fi


cat /tmp/out.$$ | grep -v ^% | grep -v ^$ > /tmp/out.$$

## OK, we are at the point of be able to look for the local object

## Replace the original source from the whois 
## with the destination source
localobj=`echo $HDL | sed "s/-${ALLOW_SOURCES[$ok]}/-${SOURCE}/"`

## Look if the handle exists in the destination source and write it a temp file 
whois -s $SOURCE -h $LOCALHOST -p $LOCALPORT -T person $localobj > /tmp/localobj.$$

## check the status
status=`grep "ERROR:101" /tmp/localobj.$$`

## If the operation was unsuccessful i.e. there is such a handle already in $SOURCE, exit
if [ "$?" == "1" ]; then

        echo -e "\n\n\tThe person handle you are looking for\n\tdoes already exist in the LOCAL DB, source ${SOURCE}.\n\n"

        exit 1

fi
	
######################################################
##
## if the object does not already exists, we can 
## go ahead. 
##
######################################################


## Write the password to the file
echo -e "password:  $PASSWORD\n\n" > /tmp/localobj.$$

## Replace the Original source by the local source
cat /tmp/out.$$ | sed "s/${ALLOW_SOURCES[$ok]}/$SOURCE/" | grep -v "^source:" | grep -v "^changed:" | grep -v "^mnt-by:" >> /tmp/localobj.$$


## Write the other attributes as well. 
echo "mnt-by:       $MNT" >> /tmp/localobj.$$
echo "changed:      $MAIL" >> /tmp/localobj.$$
echo "source:       $SOURCE" >> /tmp/localobj.$$
	
# OK. When we are here, we may create the object in the local db. 
# The use of db_wrapper avoids a complicated sudo setup with 
## config pathes and so on. 

sudo -u ipadm db_wrapper /tmp/localobj.$$

## then clean up the temp files
rm /tmp/localobj.$$
rm /tmp/out.$$
