#!/bin/sh

##################################################################################
##   db_export is used to export the content of a local Whois DB in text files	##
##										##
##   Copyright (c) 2000 - 2004 Arnd Vehling <av@nethead.de>			##
##   Copyright (c) 2003 - 3004 Tobias Cremer <ipadmin@de.cw.net			##
##   Copyright (c) 2003 - 2004 Cable & Wireless	<ripe-software.cw.net>		##
##										##
##										##
##   This file is part of ripe-ipadmin.						##
##										##
##   ripe-ipadmin is free software; you can redistribute it and/or modify	##
##   it under the terms of the GNU General Public License as published by	##
##   the Free Software Foundation; either version 2 of the License, or		##
##   (at your option) any later version.					##
##										##
##   ripe-ipadmin is distributed in the hope that it will be useful,		##
##   but WITHOUT ANY WARRANTY; without even the implied warranty of		##
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		##
##   GNU General Public License for more details.				##
##										##
##   You should have received a copy of the GNU General Public License		##
##   along with ripe-ipadmin; if not, write to the Free Software		##
##   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	##
##										##
##    $Id: db_export.sh,v 1.6 2004/12/03 20:58:06 tcremer Exp $			##
##										##
##################################################################################

. @confdir@/export.conf

## Define error funktions

function wrongsource(){

echo -e "\n\n\tThe source you've provided is not allowed. Please check your config file.\n\tAllowed sources are: \n\n\t${CONFSOURCE[@]}\n\n\tIf you have other sources configured in your Whois DB, please update the config file accordingly!\n\n\n"


exit 1

}

function error(){

echo -e "\n\n\tYou have to give the source you want to export as argument!\n\tAn optional arguments is the directory where you want store\n\tthe export files in. If DIR is omitted, the current directory is used.\n\n\tUSAGE: $0 SOURCE [DIR]"

exit 1

}

## Check arguments: A least one, not more than two. 

if [ "$#" -lt "1" -o "$#" -gt "2" ]; then

	error

fi


# Declare variables, taken from command line arguments
# SOURCE ($1) is the source as give by rip_whoisd
# DIRNAME ($2) is the directory to write files to

SOURCE="`echo $1 | awk '{ print toupper($1) }'`"
DIRNAME=$2

##############################################################
##
## Check if the first Argument is a valid DB identifier
## 
##############################################################

# Get number of sources in array
snum=${#CONFSOURCE[@]}

## Loop around and check if the source is correct
for((f=0; f < $snum; f++))
do

	if [ "$SOURCE" = "${CONFSOURCE[$f]}" ]; then

		ok=$f
	
	fi

done

# If its not in the array, display error message and exit

if [ "$ok" = '' ]; then

	wrongsource

fi

# Test the given directory argument for validity
# and handle nonexisting or empty values

if [ ".${DIRNAME}" = "." ]; then
    DIRNAME=`pwd`
elif [ -d ${DIRNAME} ]; then
    cd ${DIRNAME}
else
    mkdir ${DIRNAME}
    cd ${DIRNAME}
fi 

$TEXTEXPORT -h $CHECKHOST -P $CHECKPORT -u $CHECKUSER -p $CHECKPASSWORD $DB
chmod -f g+w ${DIRNAME}/*

# EOF
