#! @perlpath@

################################################################################
##   parsedbup parses the email send to auto-dbm@ipadm.eu.cw.net
##										
##   Copyright (c) 2000 - 2004 Arnd Vehling <av@nethead.de>			
##   Copyright (c) 2003 - 2004 Cable & Wireless	<ripe-software.cw.net>		
##										
##										
##   This file is part of ripe-ipadmin.						
##										
##   ripe-ipadmin is free software; you can redistribute it and/or modify	
##   it under the terms of the GNU General Public License as published by	
##   the Free Software Foundation; either version 2 of the License, or		
##   (at your option) any later version.					
##										
##   ripe-ipadmin is distributed in the hope that it will be useful,		
##   but WITHOUT ANY WARRANTY; without even the implied warranty of		
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		
##   GNU General Public License for more details.				
##										
##   You should have received a copy of the GNU General Public License		
##   along with ripe-ipadmin; if not, write to the Free Software		
##   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	
##										
##    $Id: parsedbup.pl,v 1.7 2004/12/03 21:00:25 tcremer Exp $			
##										
################################################################################




#    -> parsing e-mails
#    -> the script can handle more than one object at once

# Loading Modules...
use strict;
use Net::IP qw(:PROC);
use Net::RIPEWhois::in;    # Ripe Whois

use vars
  qw[ @tmp_object @opt_header @new_objects $tmp_whois $tmp_query @tmp_qlines @tmp_qlines_obj
  @config_privates %config_maintainer %config_source $my_debug
];

BEGIN: {
    require '@confdir@/parsedbup.conf';
    require '@confdir@/parsedbup.dat';
}

# Go!
&read_config;
&main;
exit;

# Finish!
sub main {
    $my_debug = 0;

    # reverse source_hash
    #
    my %config_source_tmp;
    for ( keys %config_source ) {
        $config_source_tmp{ lc( $config_source{$_} ) } = $_;
    }
    %config_source = %config_source_tmp;

    # Whois:
    #
    $tmp_whois = new RipeWhois(
        'Host'       => $Conf::whois{'lservername'},
        'Port'       => $Conf::whois{'lport'},
        'FormatMode' => 1
    );

    my $tmp_file_in  = @ARGV[0];
    my $tmp_file_out = @ARGV[1];
    my $tmp_do       = 0;          # 0 = read_from_stdin || 1 = read_from_file
    my @tmp_file;
    my $tmp_mail = 0;

    if ($tmp_file_in) {
        $tmp_do = 1;
    }
    print "[DEBUG]: tmp_file_in is $tmp_file_in\n";
    print "[DEBUG]: tmp_do is $tmp_do\n";

    if ( $tmp_do == 0 ) {
        @tmp_file = <STDIN>;
        $tmp_mail = 1;
    }
    elsif ( $tmp_do == 1 ) {
        open( FILE, $tmp_file_in );
        @tmp_file = <FILE>;
        close(FILE);
    }
    else {
        die "Internal Error: Unknown ID";
    }

    for (@tmp_file) {
        if ( ($tmp_mail) and ( $_ =~ /^\s*$/ ) ) {
            $tmp_mail = 0;
            push @opt_header, "\n";
            next;
        }    # ignore mail-header
        elsif ($tmp_mail) {
            push @opt_header, $_;
        }

        if ( $tmp_mail == 0 ) {

            # Parsing...
            if ( (/^$/) and (@tmp_object) ) {
                &parse_object;
                @tmp_object = ();
            }
            elsif (/^\S+/) {
                push @tmp_object, $_;
            }
        }    # $mail==0
    }    # for(@tmp_file)

    if (@tmp_object) {
        &parse_object;
        @tmp_object = ();
    }

    if ( @new_objects > 0 ) {
        if ($tmp_file_out) {
            open( TMP, ">$tmp_file_out" );
            for ( @opt_header, @new_objects ) {
                print TMP $_;
            }
            close(TMP);
        }
        else {
            for ( @opt_header, @new_objects ) {
                print $_;
            }
        }
    }
}

sub parse_object {
    my ( $tmp_name, $tmp_value, $range_ip1, $range_ip2 );
    my $ok       = 1;
    my $my_check = 1;
    @tmp_qlines_obj = ();

    print "[DEBUG]: parse object (first_line): $tmp_object[0]\n"
      if ( $my_debug == 1 );

    my $my_object = join( "\n", @tmp_object );
    $my_check = 0 if $my_object =~ /^delete:/m;

    if ( $my_check == 1 ) {
        for (@tmp_object) {
            if ( $_ =~ /^([^:]*):(\s|\t)*(.*)$/ ) {
                $tmp_name  = $1;
                $tmp_value = $3;

                # Replace umlaute
                $_ =~ s//ae/g;
                $_ =~ s//oe/g;
                $_ =~ s//ue/g;
                $_ =~ s//ss/g;
                $_ =~ s//Ae/g;
                $_ =~ s//Oe/g;
                $_ =~ s//Ue/g;

                if ( $1 eq 'source' ) {
                    if ( defined( $config_source{ lc($3) } ) ) {
                        $_ = "source: " . uc( $config_source{ lc($3) } ) . "\n";
                    }
                }
                elsif ( $1 eq 'inetnum' ) {
                    if ( $tmp_value =~ /(\d+\.\d+\.\d+\.\d+)\/(\d+)/ ) {
                        ( $range_ip1, $range_ip2 ) =
                          ip_prefix_to_range( $1, $2, 4 );
                        $_ = "$tmp_name: $range_ip1 - $range_ip2\n";
                    }

                    $ok = 3;
                    $tmp_whois->Query("-r -T inetnum $tmp_value");
                    $tmp_query = $tmp_whois->GetResult();
                    $tmp_query =~ s/.*?(\/copyright\.html\n\n)//s;
                    $tmp_query =~ s/^\%//m;
                    $tmp_query =~ s/\n\n\n+/\n\n/s;

                    print
"[DEBUG]: query: -r -T inetnum $tmp_value\n\n--->\n$tmp_query\n\n"
                      if ( $my_debug == 1 );

                    if ( $tmp_query =~
                        /inetnum:\s+$tmp_value.*?status:\s+assigned/is )
                    {
                        $ok = 1;
                    }
                    elsif ( $tmp_query =~ /^status:\s+assigned/im ) {
                        $ok = 3;
                    }
                    elsif ( $tmp_query =~ /^status:\s+allocated/im ) {
                        $ok = 1;
                    }
                    @tmp_qlines_obj = split( /\n/, $tmp_query );
                }
            }    # if ($_)
        }    # for (@tmp_object)
    }

    print "[DEBUG]: action: $ok\n" if ( $my_debug == 1 );
    if ( $ok == 1 ) {
        push @new_objects, @tmp_object, "\n";
    }
    else {
        &parse_error( \@tmp_object );
        print " ";
    }    # overlapping
}

sub parse_error {
    my $tmp_object = $_[0];
    my $my_to      = $Conf::mail{'mailerrors'};

    # Catch From
    if ( join( "\n", @opt_header ) =~ /^From:\s(.+)$/im ) {
        $my_to = $1;
    }

    open( MAIL, $Conf::mail{'prog'} );
    print MAIL <<END;
From: $Conf::mail{'mailerrors'}
To: $my_to
Subject: PARSE-ERROR

The inetnum object:

END

    for (@$tmp_object) {
        print MAIL $_;
    }

    print MAIL <<END;

would overwrite the following inetnum object(s):

END

    for (@tmp_qlines_obj) {
        print MAIL "$_\n";
    }

    print MAIL <<END;

Please delete the inetnum if you really want to
replace it with a new inetnum-object.

Sincerly,

        Your Automatic Database Department
END
    close(MAIL);
}

