#!@perlpath@

##################################################################################
##   ripadm is used to sync a local Whois DB with it's RIRs counterpart		##
##										##
##   Copyright (c) 2000 - 2004 Arnd Vehling <av@nethead.de>			##
##   Copyright (c) 2003 - 2004 Cable & Wireless	<ripe-software.cw.net>		##
##										##
##										##
##										##
##   This file is part of ripe-ipadmin.						##
##										##
##   ripe-ipadmin is free software; you can redistribute it and/or modify	##
##   it under the terms of the GNU General Public License as published by	##
##   the Free Software Foundation; either version 2 of the License, or		##
##   (at your option) any later version.					##
##										##
##   ripe-ipadmin is distributed in the hope that it will be useful,		##
##   but WITHOUT ANY WARRANTY; without even the implied warranty of		##
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		##
##   GNU General Public License for more details.				##
##										##
##   You should have received a copy of the GNU General Public License		##
##   along with ripe-ipadmin; if not, write to the Free Software		##
##   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA	##
##										##
##    $Id: ripadm.pl,v 1.7 2004/12/03 09:22:37 tcremer Exp $			##
##										##
##################################################################################



 
# /////////////////////////////////////////////////////////////////////////////
# Filename          : ripadm.pl
# Purpose           : 
#                     
# Author            : NetHead / info@nethead.de / http://nethead.de
#		     
# Description       :  
# OSs Tested        : Linux, SunOS 5.8
# Command Line      : starting without options will give you help
#             
# Comments          : Synchronizes the RIPE-DB with the local DB.
#                   : Checks per NRTM protocol for new or changed 
#                   : objects in the local db and creates according
#                   : update mails for updating the ripe-registry
#                   : See also "reply.pl"
#
# Version           : 1.0
# Date:             : 2001-12-07
# /////////////////////////////////////////////////////////////////////////////


use strict;

## This is the old line, update below with new path to Telnet.pm
#require "/cw/lib/perl5/5.6.1/Net/Telnet.pm";
#require "/v/ipadm/sw/lib/perl/vendor_perl/5.8.4/Net/Telnet.pm"; 


## Due to ongoing update problems do not use hardcoded path to Telnet.pm
## This object oriented call is not really tested, just hoping is works all fine

use Net::Telnet;

use vars qw[ 	@my_object $my_object_typ
		@config_privates %config_maintainer %config_source
		$current_id $int_stop $auth
		$tmp_ripedb
		$use_config @config_sources
           ];

###
### $current_id ist die ID vom Objekt, welches als naechstes bearbeitet werden sollte 
###

# Exit/Status-Codes:
#
# 0 -> OK
# 1 -> Interrupt
# 2 -> Config-File not found
# 3 -> PID-File found
# 4 -> Start-ID greater than NRTM-Start-ID
# 5 -> Unknown Source
# 6 -> Lokal-Start < NRTM-Start
# 7 -> unknown NRTM-Objekttyp
# 8 -> unknown data
# 9 -> No Maintainer PW

# 50 -> Connection failed
# 51 -> Connection timed out
# 52 -> No Prompt

BEGIN: 	{require '@confdir@/ripadm.dat';
	# config-file 
	$use_config = '@confdir@/ripadm.conf'; 
	for (@ARGV) 	{
			if ($_ =~ /\-\-config:(.*)/) 	{
				if (-e $1) 	{$use_config = $1} 
				else 
						{require $use_config; ripadm_fehler("Config-File not found: $1",'ERR-2',0,__FILE__,__LINE__,2);}
							}
			} 
	require $use_config;	
	}

	$int_stop = 0;
	$SIG{INT} = sub {$int_stop = 1};

&checkrun;
&read_config;
&main;
&stopping;
exit 0;

#
# Check if we are already running
#

sub checkrun
{
        if ($Conf::debug>1)
        {&message("Checking if ($$) is allready runnuing ... $Conf::file{'pid'}")}

	if (-e $Conf::file{'pid'}) {ripadm_fehler('Program activ? Check PID-File!','ERR-3',0,__FILE__,__LINE__,3)}
	else
	{
	open(PID,">$Conf::file{'pid'}");
	print PID $$;
	close(PID);
	}
}

######
###
######
#
# Delete PID-File 
#

sub stopping
{
	if ($Conf::debug>1)
        {&message("Deleting PID-FILE... $Conf::file{'pid'}")}
	unlink($Conf::file{'pid'});
}

#
# Hauptroutine
#

sub main
{
	# Init...
        my $my_obj_end = $current_id + $Conf::nrtm{'max'};

#
# Kommandozeile
#

        my $cmd_stop = 0;
	my $nrtm_info = 0;
	my $cmd_sync = 0;
	my $cmd_status = 0;

	for (@ARGV)
	{	

        if ($_ eq '--all')		# Alle Objekte bearbeiten
        {
	$my_obj_end = 'LAST';
	}
	elsif ($_ =~ /^\-\-start:(\d+)$/)	# Start-ID festlegen
	{
	$current_id = $1; $my_obj_end = $current_id + $Conf::nrtm{'max'};
	}
        elsif ($_ =~ /^\-\-stop:(\d+)$/) # Stop-ID festlegen
        {
        $my_obj_end = $1;
        }
	elsif ($_ =~ /^\-\-debug:(\d+)$/)  # Debugmodus aktivieren 
        {
	$Conf::debug = $1; 
	}
        elsif ($_ =~ /^\-\-mntstop:(\d+)$/)  # Was tun, bei auth-probs 
        {
        $Conf::debug_config{'stop_at_nopwd'} = $1;
        }
        elsif ($_ =~ /^\-\-sendby:(\d+)$/)  # Objekte per STDOUT oder MAIL 
        {
        $Conf::debug_config{'sendby'} = $1;
        }
	elsif ($_ =~ /^\-\-source:([A-Za-z]*)$/)
	{
	my $tmp_source_ok = 0;
	my $dummy;
		foreach $dummy (@config_sources)
		{
		if ($dummy eq $1) {$tmp_source_ok = 1; $config_sources[0] = $dummy}
		}
	if ($tmp_source_ok == 0) {&stopping; ripadm_fehler("Unkown Source $1",'ERR-5',0,__FILE__,__LINE__,5);}
	}
        elsif ($_ eq '--nrtm') {$nrtm_info = 1; $cmd_sync = 1} 
	elsif ($_ eq '--sync') {$cmd_sync = 1}
        elsif ($_ =~ /^\-\-config:.*$/) {}        # too late ;-) see to
	elsif ($_ eq '--status') {$cmd_status = 1}    
        else    {$cmd_sync = 0}

        } # for (@ARGV)


	if ($cmd_status)	
	{

	formline <<END, $current_id, $my_obj_end, $Conf::nrtm{'max'};

NRTM-Serial-Range
-----------------
Start-ID			: @>>>>>> 
Stop-ID				: @>>>>>> 

NRTM-Data (see ripeadm.conf)
----------------------------
Objects per script-run          : @>>>>>>
Servername			: $Conf::nrtm{'servername'}
Port				: $Conf::nrtm{'port'}
Timeout				: $Conf::nrtm{'timeout'} sec.
Default-Source			: $config_sources[0]

END
	print $^A;

#Runtime Variables
#-----------------
#Debugmode			: $Conf::debug
#Script will stop on non auth  	: $Conf::debug_config{'stop_at_nopwd'}
#Objects goesen an 0 STDOUT	: $Conf::debug_config{'sendby'}

	$cmd_stop = 1;
		}

        if (($cmd_sync == 0) and ($cmd_stop == 0)) {                   # Hilfestellung
print <<END;

--status           Displays various script and server variables 
--nrtm             Display start and stop ids of the nrtm server
--all              Process ALL objects which changed since last run
--start:<num>      Set start id to <num>
--stop:<num>       Set stop id to <num>
--debug:<mode>     Debugmode: 1 = short output, 2 = long output
--mntstop:<num>    Stop if an unkown maintainer is referrenced.
                   The default is to continue and send error mails.
--sendby:<num>     0 = send to STDOUT,  1 = send per e-mail (default)
--sync             Start synchronisation as configured via config files.
--config:<file>	   Use alternativ config file <file>
--source:<Source>  Sync using <Source> (see config.dat)

END
        $cmd_stop = 1;
                }

	if ($cmd_stop) {&stopping; exit 0;}

###
### ReInit...
###

        my $my_current_line;	# Zeilen NRTM-Connect
        my $tmp_status = 0;	# NRTM-Parser
        my $my_obj_recv_start;	# Startwert NRTM - beim Ausfhren
        my $my_obj_recv_end;	# Stopwert NRTM	- beim Ausfhren
	my $my_obj_ask_start = 0;  # Startwert NRTM - vor Ausfhren
        my $my_obj_ask_end = 0;    # Stopwert NRTM - vor Ausfhren

	$tmp_ripedb=&server_connect(0);

###
### Sending Command
###

        my $my_cmd = "-q sources";
	if ($Conf::debug>1)
        {&message("Sende Befehl: $my_cmd")}
        $tmp_ripedb->print($my_cmd);

        while ($my_current_line=$tmp_ripedb->getline(Timeout => $Conf::nrtm{'timeout'}))
	{
	chomp $my_current_line;
	if ($my_current_line =~ /^$config_sources[0]:$Conf::nrtm{'pversion'}:Y:(\d+)\-(\d+)$/) {$my_obj_ask_start = $1; $my_obj_ask_end = $2;}
	}

	if ($nrtm_info)
	{

formline <<END, $current_id, $my_obj_end, $my_obj_ask_start, $my_obj_ask_end;

### NRTM ###
LOKAL-Start-ID			: @>>>>>>
LOKAL-Stop-ID			: @>>>>>>
-
NRTM-Start-ID			: @>>>>>>
NRTM-Stop-ID			: @>>>>>>
###

END
print $^A;

	&stopping; exit 0;
	}


        if ($Conf::debug>0)
        {
	&message("LOKAL-Start-ID: $current_id LOKAL-Stop-ID: $my_obj_end");
	&message("NRTM-Start-ID: $my_obj_ask_start NRTM-Stop-ID: $my_obj_ask_end");
	}

###
### Checken der Start/Stop-Werte
###


	if ($current_id<$my_obj_ask_start) {&stopping; ripadm_fehler("Lokal-Start-ID ($current_id) ist kleiner als NRTM-Start-ID ($my_obj_ask_start)",'ERR-6',0,__FILE__,__LINE__,6)}
	if (($my_obj_end ne 'LAST') and ($my_obj_end>$my_obj_ask_end)) {$my_obj_end = $my_obj_ask_end;  if ($Conf::debug>1) {&message("Korrigiere NRTM-Stop-ID, da lokale Stop-ID hher")}}
	if ($current_id > $my_obj_ask_end) {&stopping; exit 4;}

        $tmp_ripedb->close; 

	$tmp_ripedb=&server_connect(0);

	$my_cmd = "-g $config_sources[0]:$Conf::nrtm{'pversion'}:$current_id:$my_obj_end";
        if ($Conf::debug>1)
        {&message("Sende Befehl: $my_cmd")}
	$tmp_ripedb->print($my_cmd);

	my $tmp_start = 0;

# $tmp_status:
#
# 0 -> eine Aktion ADD/UPDATE/DELETE folgt
# 1 -> Leerzeile + ein Objekt folgt 
# 2 -> ein Objekt folgt

	

	while ( $my_current_line=$tmp_ripedb->getline(Timeout => $Conf::nrtm{'timeout'}) )
	{
	chomp $my_current_line; 
	if (!$tmp_start)
	{
	if ($my_current_line !~ /^%START Version: \d [a-zA-Z0-9]* (\d+)\-(\d+)$/) {next} else {$my_obj_recv_start = $1; $my_obj_recv_end = $2; $tmp_start = 1; next;}
	}

###
### Recieving Objects
###

	if ($tmp_status == 0)
	{
	if 	($my_current_line =~ /^\s*$/) {next}
	elsif	($my_current_line =~ /^%END/) {last}	
	elsif 	($my_current_line =~ /^(ADD|DEL)$/) {$my_object_typ = $1; $tmp_status = 1; next}
	else {&stopping; ripadm_fehler("Kann Objekttyp nicht zuordnen: !$my_current_line!",'ERR-7',0,__FILE__,__LINE__,7)}
	}

	elsif ($tmp_status == 1)
	{
	if      ($my_current_line =~ /^\s*$/) {$tmp_status = 2; next}
	else {&stopping; ripadm_fehler('Eingabeformat stimmt nicht','ERR-8',0,__FILE__,__LINE__,8)}
	}

	elsif ($tmp_status == 2)
        {
	if      ($my_current_line =~ /^\s*$/) {&ripadm_lastobject; $tmp_status = 0; next}
	push @my_object, $my_current_line;
	}

###
###
###
	};

	&ripadm_lastobject;	# LASTRUN

	$tmp_ripedb->close;
	print "Script terminated normally\n";
}

######
###
######

sub message
{

	my ($t_sec, $t_min, $t_std, $t_tag, $t_mon, $t_jahr, undef, undef, undef) = localtime(time);
	$t_mon++; if ($t_mon == 13) {$t_mon = 1}
	$t_jahr += 1900;
	for ($t_sec, $t_min, $t_std, $t_tag, $t_mon)
	{
	$_ =~ s/^(\d)$/0$1/;
	}
	
	my $date = "$t_jahr-$t_mon-$t_tag $t_std:$t_min:$t_sec";
	if ($Conf::file{'log'} ne '')
	{
	open(LOG,">>$Conf::file{'log'}");
	print LOG "$date $_[0]\n";	
	close(LOG);
	}
	else
	{
	print "$date $_[0]\n";
	}
}

sub ripadm_lastobject
{
	# Init...
	my $tmp_drop = 0;
	my $tmp_export = 1;
	my $tmp_obj;
	my $x;
	my @tmp_object;
	my $tmp_dropper;
	my $tmp_typ;
	my $tmp_wert;
	$auth = '';
	my $my_source;
	my $my_ismnt;
	###

	if (@my_object)
	{

	if ($my_object[0] =~ /^([^:]*):\s*(.*)/) {if (&is_private($1)) {$tmp_drop = 1}; $tmp_typ = $1; $tmp_wert=$2 }


	if ($Conf::obj_export{$tmp_typ} ne '') {$tmp_export = 0}

	# vorher erstmal source suchen...
	for (@my_object) { if ($_ =~ /^source:(\s|\t)*(.*)$/) { $my_source = lc($2) } }

	for (@my_object)
	{
###
### Objekt zeilenweise parsen...
###

		if ($_ =~ /^([^:]*):(\s|\t)*(.*)$/) 
		{
		if (&is_private($1)) {next}
		if ($1 eq 'mnt-by') 
			{
			# authorisieren...
			if ($Conf::debug>2) {&message("Maintainer lokalisiert: $3")}
			$my_ismnt = 1;
			if (defined($config_maintainer{lc($3)})) {$auth = $config_maintainer{lc($3)}}
			}
		if ($1 eq 'source')
			{
                        if (defined($config_source{lc($3)})) {$_ = "source: $config_source{lc($3)}"} 
			}
		}

		foreach $tmp_dropper (@Conf::obj_drop)
		{
		if ($_ =~ /$tmp_dropper/i) {$tmp_drop = 1} 
		}

		if ($Conf::obj_export{$tmp_typ} ne '') 
		{
		if ($_ =~ /$Conf::obj_export{$tmp_typ}/i) {$tmp_export = 1}
		}

		if ( ($my_source ne '') and (defined($config_source{$my_source})) and ($_ =~ /\-$my_source$/i) ) {$_ =~ s/\-$my_source$/-$config_source{$my_source}/i} 	

		push @tmp_object, $_;	

	} # for (@my_object)

	@my_object = @tmp_object;

	
        if ($Conf::debug>1)
        {&message("Drop-Status: $tmp_drop Export-Status: $tmp_export")}	

	if (($my_ismnt == 1) and ($auth eq '') and ($Conf::debug_config{'stop_at_nopwd'})) {&stopping; ripadm_fehler("Kein Kennwort fr einen Maintainer vom Objekt gefunden",'ERR-9',0,__FILE__,__LINE__,9)}

	if (($tmp_drop == 0) and ($tmp_export == 1)) 
		{&message("ID: $current_id $my_object_typ $tmp_typ: $tmp_wert EXPORT: $Conf::dir{'objects_send'}/$current_id.$my_object_typ.$$"); &ripemail} else 
		{&message("ID: $current_id $my_object_typ $tmp_typ: $tmp_wert DROP")}

	&dec_object_id;

	} # if (@my_object)

	@my_object = ();
	$my_object_typ = '';

###
### Stop Security
###

	if ($int_stop == 1) {&stopping; print "Script wurde erfolgreich angehalten\n"; exit 1;} 

}

######
###
######

sub ripemail
{


###
### Text vorbereiten
###

	my $dummy_header;
	my $dummy_body;

$dummy_header = <<END;
Message-Id: $current_id.$my_object_typ.$$\@ipadm0.eu.cw.net
Message-Id: $current_id.$my_object_typ.$$\@ipadm0.eu.cw.net
From: $Conf::mail{'from'}
To: $Conf::mail{'to'}

END

	if ($auth ne '') {$dummy_body .= "password: $auth\n"}
	
        for (@my_object)
        {
	$dummy_body .= "$_\n";
	}

	if ($my_object_typ eq 'DEL') 
	{
	$dummy_body .= "delete: $Conf::mail{'from'}\n"
	}

###
### Sendmail
### 

	if ($Conf::debug_config{'sendby'}==1)
	{
        open(MAIL,$Conf::mail{'prog'});
	print MAIL $dummy_header . $dummy_body; 
	close(MAIL);
	} else
	{
	print "$dummy_body\n";
	}


###
### Savemail
###

	open(OBJ,">$Conf::dir{'objects_send'}/$current_id.$my_object_typ.$$");
	print OBJ $dummy_body;
	close(OBJ);

}

######
###
######

sub dec_object_id
{
	$current_id++;
	open(OBJC,">$Conf::file{'current_id'}");
	print OBJC $current_id;
	close(OBJC);
}

sub ripadm_fehler
{
        my ($my_msg, $my_code, $my_status, $my_file, $my_line, $my_exit) = @_;

	if ($Conf::debug)
	{
	&message("Error: $my_msg, $my_code, $my_status, $my_file, $my_line (Objekt: $current_id)");
	}

	if ($Conf::mail{'mailerrors'} ne '')
 	{
	open(MAIL,$Conf::mail{'prog'});
        print MAIL <<END;
From: $Conf::mail{'mailerrors'}
To: $Conf::mail{'mailerrors'}
Subject: RIPADM-ERROR

Error:
Message: $my_msg
Code: $my_code
Status: $my_status
Aktuelles Objekt: $current_id
File: $my_file
Line: $my_line

Anbei - falls vorhanden - das aktuelle Objekt:

END


	for (@my_object)
	{
	print MAIL "$_\n";
	}

	close(MAIL);
	} else
	{
        print <<END;
Error:
Message: $my_msg
Code: $my_code
Status: $my_status
Aktuelles Objekt: $current_id
File: $my_file
Line: $my_line
END
	}

        exit $my_exit;
}

